/*
 * Copyright (C) 2019 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package android.content.pm;

import android.annotation.NonNull;
import android.annotation.Nullable;
import android.annotation.SystemApi;
import android.content.Intent;
import android.os.Parcelable;
import android.os.UserHandle;

import com.android.internal.util.DataClass;

/**
 * Information exposed to {@link android.app.InstantAppResolverService} to complete
 * an instant application resolution request.
 * @hide
 */
@SystemApi
@DataClass(genParcelable = true, genConstructor = true, genAidl = true)
public final class InstantAppRequestInfo implements Parcelable {

    /**
     * The sanitized {@link Intent} used for resolution. A sanitized Intent is an intent with
     * potential PII removed from the original intent. Fields removed include extras and the
     * host + path of the data, if defined.
     */
    @NonNull
    public final Intent intent;

    /** The hash prefix of the instant app's domain or null if no host is defined. */
    @Nullable
    public final int[] hostDigestPrefix;

    /** The user requesting the instant application */
    @NonNull
    public final UserHandle userHandle;

    /** Whether or not the requesting package was an instant app itself */
    public final boolean isRequesterInstantApp;

    /** A unique identifier */
    @NonNull
    public final String token;



    // Code below generated by codegen v1.0.13.
    //
    // DO NOT MODIFY!
    // CHECKSTYLE:OFF Generated code
    //
    // To regenerate run:
    // $ codegen $ANDROID_BUILD_TOP/frameworks/base/core/java/android/content/pm/InstantAppRequestInfo.java
    //
    // To exclude the generated code from IntelliJ auto-formatting enable (one-time):
    //   Settings > Editor > Code Style > Formatter Control
    //@formatter:off


    /**
     * Creates a new InstantAppRequestInfo.
     *
     * @param intent
     *   The sanitized {@link Intent} used for resolution. A sanitized Intent is an intent with
     *   potential PII removed from the original intent. Fields removed include extras and the
     *   host + path of the data, if defined.
     * @param hostDigestPrefix
     *   The hash prefix of the instant app's domain or null if no host is defined.
     * @param userHandle
     *   The user requesting the instant application
     * @param isRequesterInstantApp
     *   Whether or not the requesting package was an instant app itself
     * @param token
     *   A unique identifier
     */
    @DataClass.Generated.Member
    public InstantAppRequestInfo(
            @NonNull Intent intent,
            @Nullable int[] hostDigestPrefix,
            @NonNull UserHandle userHandle,
            boolean isRequesterInstantApp,
            @NonNull String token) {
        this.intent = intent;
        com.android.internal.util.AnnotationValidations.validate(
                NonNull.class, null, intent);
        this.hostDigestPrefix = hostDigestPrefix;
        this.userHandle = userHandle;
        com.android.internal.util.AnnotationValidations.validate(
                NonNull.class, null, userHandle);
        this.isRequesterInstantApp = isRequesterInstantApp;
        this.token = token;
        com.android.internal.util.AnnotationValidations.validate(
                NonNull.class, null, token);

        // onConstructed(); // You can define this method to get a callback
    }

    @Override
    @DataClass.Generated.Member
    public void writeToParcel(@NonNull android.os.Parcel dest, int flags) {
        // You can override field parcelling by defining methods like:
        // void parcelFieldName(Parcel dest, int flags) { ... }

        byte flg = 0;
        if (isRequesterInstantApp) flg |= 0x8;
        if (hostDigestPrefix != null) flg |= 0x2;
        dest.writeByte(flg);
        dest.writeTypedObject(intent, flags);
        if (hostDigestPrefix != null) dest.writeIntArray(hostDigestPrefix);
        dest.writeTypedObject(userHandle, flags);
        dest.writeString(token);
    }

    @Override
    @DataClass.Generated.Member
    public int describeContents() { return 0; }

    /** @hide */
    @SuppressWarnings({"unchecked", "RedundantCast"})
    @DataClass.Generated.Member
    /* package-private */ InstantAppRequestInfo(@NonNull android.os.Parcel in) {
        // You can override field unparcelling by defining methods like:
        // static FieldType unparcelFieldName(Parcel in) { ... }

        byte flg = in.readByte();
        boolean _isRequesterInstantApp = (flg & 0x8) != 0;
        Intent _intent = (Intent) in.readTypedObject(Intent.CREATOR);
        int[] _hostDigestPrefix = (flg & 0x2) == 0 ? null : in.createIntArray();
        UserHandle _userHandle = (UserHandle) in.readTypedObject(UserHandle.CREATOR);
        String _token = in.readString();

        this.intent = _intent;
        com.android.internal.util.AnnotationValidations.validate(
                NonNull.class, null, intent);
        this.hostDigestPrefix = _hostDigestPrefix;
        this.userHandle = _userHandle;
        com.android.internal.util.AnnotationValidations.validate(
                NonNull.class, null, userHandle);
        this.isRequesterInstantApp = _isRequesterInstantApp;
        this.token = _token;
        com.android.internal.util.AnnotationValidations.validate(
                NonNull.class, null, token);

        // onConstructed(); // You can define this method to get a callback
    }

    @DataClass.Generated.Member
    public static final @NonNull Parcelable.Creator<InstantAppRequestInfo> CREATOR
            = new Parcelable.Creator<InstantAppRequestInfo>() {
        @Override
        public InstantAppRequestInfo[] newArray(int size) {
            return new InstantAppRequestInfo[size];
        }

        @Override
        public InstantAppRequestInfo createFromParcel(@NonNull android.os.Parcel in) {
            return new InstantAppRequestInfo(in);
        }
    };

    @DataClass.Generated(
            time = 1574373347443L,
            codegenVersion = "1.0.13",
            sourceFile = "frameworks/base/core/java/android/content/pm/InstantAppRequestInfo.java",
            inputSignatures = "public final @android.annotation.NonNull android.content.Intent intent\npublic final @android.annotation.Nullable int[] hostDigestPrefix\npublic final @android.annotation.NonNull android.os.UserHandle userHandle\npublic final  boolean isRequesterInstantApp\npublic final @android.annotation.NonNull java.lang.String token\nclass InstantAppRequestInfo extends java.lang.Object implements [android.os.Parcelable]\n@com.android.internal.util.DataClass(genParcelable=true, genConstructor=true, genAidl=true)")
    @Deprecated
    private void __metadata() {}


    //@formatter:on
    // End of generated code

}
